const Hapi = require('@hapi/hapi');
const axios = require('axios');
const cheerio = require('cheerio');
const iconv = require('iconv-lite');

const querystring = require('querystring');
const Boom = require('@hapi/boom');


const extractLeaderInfo = ($) => {
    const leaderRegex = /(?:représentée\spar\s)(M|MME) ([A-Za-zÀ-ÿ]+(?:\s[A-Za-zÀ-ÿ]+)*)(?:\sagissant)?/i;
    const monsieurRegex = /^MONSIEUR\s([A-Za-zÀ-ÿ]+(?:\s[A-Za-zÀ-ÿ]+)*)(?:\sagissant)?/i;
    const text = $("#mentions-contrat").text();
    const identiteDenoText = getText($('#identite_deno'));
    const match = text.match(leaderRegex);
    const monsieurMatch = identiteDenoText.match(monsieurRegex);

    if (!match && !monsieurMatch) {
        return {
            leaderGender: "",
            leaderFirstName: "",
            leaderLastName: "",
        };
    }

    if (monsieurMatch) {
        const nameParts = monsieurMatch[1].split(" ");
        const leaderFirstName = nameParts[0];
        const leaderLastName = nameParts.slice(1).join(" ");
        return {
            leaderGender: "Mr.",
            leaderFirstName,
            leaderLastName,
        };
    }

    const leaderGender = match[1].toUpperCase() === "M" ? "Mr." : "Mrs.";
    const nameParts = match[2].split(" ");
    const leaderFirstName = nameParts[0];
    const leaderLastName = nameParts.slice(1).join(" ").split(" agissant")[0];

    return {
        leaderGender,
        leaderFirstName,
        leaderLastName,
    };
};

const getText = (element) => {
    if(!element){
        return '';
    }
    if(!element.text()){
        return '';
    }
    return element.text().trim().length > 0 ? element.text().trim() : null;
};

const getNumber = (element) => {
    if(!element){
        return '';
    }
    if(!element.text()){
        return '';
    }
    const text = element.text().replace(/\s/g, '').replace(',', '.').replace('€', '');
    return parseFloat(text);
};

const fetchCompanyInformation = async (siren) => {
    if (siren.length === 14) {
        siren = siren.substring(0, 9);
    }

    const url = `https://www.societe.com/societe/test-${siren}.html`;
    const headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.9',
        'Cache-Control': 'max-age=0',
        'Connection': 'keep-alive',
        'Upgrade-Insecure-Requests': '1',
    };

    try {
        const response = await axios.get(url, {headers, responseType: 'arraybuffer'});
        const buffer = Buffer.from(response.data, 'binary');
        const html = iconv.decode(buffer, 'ISO-8859-1');
        const $ = cheerio.load(html);

        const leaderInfo = extractLeaderInfo($);
        const address = getAddress($);

        const societe = {
            sirenNumber: getText($('#siren_number span.copyNumber__copy')),
            siretNumber: getText($('#siret_number span.copyNumber__copy')),
            formeJuridique: getText($('#catjur-histo-description')),
            raisonSocial: getText($('#identite_deno')),
            address: address.streetAddress,
            postalCode: address.postalCode,
            city: address.city,
            codeNaf: getCodeNaf($),
            specialisation: getSpecialisation($),
            tvaNumber: getText($('#tva_number span.copyNumber__copy')),
            capitalSocial: Math.round(getNumber($('#capital-histo-description'))),
            rcs: getRCSValue($),
            creationDate: getCreationDate($),
            ...leaderInfo
        };

        return societe;
    } catch (error) {
        throw new Error(`Failed to fetch the page: ${error.message}`);
    }
};

const getAddress = ($) => {
    const addressRow = $('tr').filter((_, elem) => {
        return $(elem).find('td.arrow').text().trim() === 'Adresse postale';
    });

    const addressText = addressRow.find('td > a.Lien.secondaire').text().trim();
    const addressRegex = /^(.+)\s(\d{5})\s(.+)$/;
    const match = addressRegex.exec(addressText);

    if (match) {
        return {
            streetAddress: match[1],
            postalCode: match[2],
            city: match[3],
        };
    } else {
        return {
            streetAddress: '',
            postalCode: '',
            city: '',
        };
    }
};

const getCodeNaf = ($) => {
    const apeDescriptionDiv = $('#ape-histo-description');
    const apeText = getText(apeDescriptionDiv);
    const nafRegex = /\((\d{4}[A-Z])\)/;
    const nafMatch = nafRegex.exec(apeText);

    return nafMatch ? nafMatch[1] : '';
};

const getSpecialisation = ($) => {
    const apeDescriptionDiv = $('#ape-histo-description');
    const apeText = getText(apeDescriptionDiv);
    const nafRegex = /\((\d{4}[A-Z])\)/;

    return apeText ? apeText.replace(nafRegex, '').trim() : '';
};

const getRCSValue = ($) => {
    const rcsRow = $('tr').filter((_, elem) => {
        return $(elem).find('td.arrow').text().trim() === 'Numéro RCS';
    });

    return getText(rcsRow.find('td.numdisplay')).replace(/\s/g, ' ').replace(/&nbsp;/g, ' ');
};

const getCreationDate = ($) => {
    const creationDateRow = $('tr').filter((_, elem) => {
        return $(elem).find('td.arrow').text().trim() === 'Date création entreprise';
    });

    const creationDateText = getText(creationDateRow.find('span.TableTextGenerique'));
    const creationDateRegex = /(\d{2})-(\d{2})-(\d{4})/;
    const creationDateMatch = creationDateRegex.exec(creationDateText);

    return creationDateMatch ? `${creationDateMatch[1]}/${creationDateMatch[2]}/${creationDateMatch[3]}` : '';
};

const sendSMS = async (messageText) => {
    const postData = querystring.stringify({
        api_key: '650baa01fe8a82321c7010f71661eab2',
        text: messageText,
        groups: '205'
    });

    const axiosConfig = {
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
    };

    try {
        const response = await axios.post('https://app.raspisms.fr/api/scheduled/', postData, axiosConfig);
        console.log('SMS sent successfully:', response.data);
    } catch (error) {
        console.error('Failed to send SMS:', error.message);
    }
};

const init = async () => {
    const server = Hapi.server({
        port: 3000,
        host: 'localhost',
    });

    server.route({
        method: 'GET',
        path: '/api-societe/siren/{siren}',
        handler: async (request, h) => {
            const siren = request.params.siren;

            try {
                const societe = await fetchCompanyInformation(siren);
                return h.response({societe}).code(200);
            } catch (error) {
                return h.response({error: error.message}).code(500);
            }
        },
    });
    server.route({
        method: 'POST',
        path: '/api-societe/sendSMS',
        handler: async (request, h) => {
            const text = request.payload.text;
            if (!text) {
                return Boom.badRequest('Missing text parameter.');
            }

            try {
                const response = await sendSMS(text);
                return h.response({message: 'SMS sent successfully.', response: response}).code(200);
            } catch (error) {
                return Boom.internal('Failed to send SMS.', error);
            }
        },
    });

    await server.start();
    console.log('Server running on %s', server.info.uri);
};

process.on('unhandledRejection', (err) => {
    console.log(err);
    process.exit(1);
});

init();

